@@ -12,7 +12,7 @@ except ImportError:
 from time import sleep
 from itertools import repeat, cycle, chain
 from threading import Lock
-from collections import OrderedDict
+from collections import OrderedDict, Counter
 
 from .exc import (
     DeviceClosed,
@@ -29,6 +29,7 @@ from .output_devices import (
     RGBLED,
     Buzzer,
     Motor,
+    PhaseEnableMotor,
     )
 from .threads import GPIOThread
 from .devices import Device, CompositeDevice
@@ -47,6 +48,10 @@ class CompositeOutputDevice(SourceMixin, CompositeDevice):
         specific order). All keyword arguments *must* be included in the
         collection. If omitted, an alphabetically sorted order will be selected
         for keyword arguments.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
     """
 
     def on(self):
@@ -124,6 +129,10 @@ class ButtonBoard(HoldMixin, CompositeDevice):
         executed once per hold. This parameter can only be specified as a
         keyword parameter.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     :param \*\*named_pins:
         Specify GPIO pins that buttons of the board are attached to,
         associating each button with a property name. You can designate as
@@ -135,6 +144,7 @@ class ButtonBoard(HoldMixin, CompositeDevice):
         bounce_time = kwargs.pop('bounce_time', None)
         hold_time = kwargs.pop('hold_time', 1)
         hold_repeat = kwargs.pop('hold_repeat', False)
+        pin_factory = kwargs.pop('pin_factory', None)
         order = kwargs.pop('_order', None)
         super(ButtonBoard, self).__init__(
             *(
@@ -142,6 +152,7 @@ class ButtonBoard(HoldMixin, CompositeDevice):
                 for pin in args
                 ),
             _order=order,
+            pin_factory=pin_factory,
             **{
                 name: Button(pin, pull_up, bounce_time, hold_time, hold_repeat)
                 for name, pin in kwargs.items()
@@ -205,24 +216,31 @@ class LEDCollection(CompositeOutputDevice):
     :class:`LEDBoard` and :class:`LEDBarGraph`.
     """
     def __init__(self, *args, **kwargs):
-        self._blink_thread = None
         pwm = kwargs.pop('pwm', False)
         active_high = kwargs.pop('active_high', True)
         initial_value = kwargs.pop('initial_value', False)
+        pin_factory = kwargs.pop('pin_factory', None)
         order = kwargs.pop('_order', None)
         LEDClass = PWMLED if pwm else LED
         super(LEDCollection, self).__init__(
             *(
                 pin_or_collection
                 if isinstance(pin_or_collection, LEDCollection) else
-                LEDClass(pin_or_collection, active_high, initial_value)
+                LEDClass(
+                    pin_or_collection, active_high, initial_value,
+                    pin_factory=pin_factory
+                )
                 for pin_or_collection in args
                 ),
             _order=order,
+            pin_factory=pin_factory,
             **{
                 name: pin_or_collection
                 if isinstance(pin_or_collection, LEDCollection) else
-                LEDClass(pin_or_collection, active_high, initial_value)
+                LEDClass(
+                    pin_or_collection, active_high, initial_value,
+                    pin_factory=pin_factory
+                )
                 for name, pin_or_collection in kwargs.items()
                 })
         leds = []
@@ -283,6 +301,10 @@ class LEDBoard(LEDCollection):
         the device will be switched on initially. This parameter can only be
         specified as a keyword parameter.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     :param \*\*named_pins:
         Specify GPIO pins that LEDs of the board are attached to, associating
         each LED with a property name. You can designate as many pins as
@@ -291,12 +313,16 @@ class LEDBoard(LEDCollection):
         create trees of LEDs.
     """
     def __init__(self, *args, **kwargs):
+        self._blink_thread = None
         self._blink_leds = []
         self._blink_lock = Lock()
         super(LEDBoard, self).__init__(*args, **kwargs)
 
     def close(self):
-        self._stop_blink()
+        try:
+            self._stop_blink()
+        except AttributeError:
+            pass
         super(LEDBoard, self).close()
 
     def on(self, *args):
@@ -486,6 +512,10 @@ class LEDBarGraph(LEDCollection):
         The initial :attr:`value` of the graph given as a float between -1 and
         +1.  Defaults to ``0.0``. This parameter can only be specified as a
         keyword parameter.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
     """
 
     def __init__(self, *pins, **kwargs):
@@ -495,9 +525,12 @@ class LEDBarGraph(LEDCollection):
         pwm = kwargs.pop('pwm', False)
         active_high = kwargs.pop('active_high', True)
         initial_value = kwargs.pop('initial_value', 0.0)
+        pin_factory = kwargs.pop('pin_factory', None)
         if kwargs:
             raise TypeError('unexpected keyword argument: %s' % kwargs.popitem()[0])
-        super(LEDBarGraph, self).__init__(*pins, pwm=pwm, active_high=active_high)
+        super(LEDBarGraph, self).__init__(
+            *pins, pwm=pwm, active_high=active_high, pin_factory=pin_factory
+        )
         try:
             self.value = initial_value
         except:
@@ -549,6 +582,22 @@ class LEDBarGraph(LEDCollection):
         for index, led in enumerate(leds):
             led.value = calc_value(index)
 
+    @property
+    def lit_count(self):
+        """
+        The number of LEDs on the bar graph actually lit up. Note that just
+        like ``value``, this can be negative if the LEDs are lit from last to
+        first.
+        """
+        lit_value = self.value * len(self)
+        if not isinstance(self[0], PWMLED):
+            lit_value = int(lit_value)
+        return lit_value
+
+    @lit_count.setter
+    def lit_count(self, value):
+        self.value = value / len(self)
+
 
 class LedBorg(RGBLED):
     """
@@ -572,12 +621,17 @@ class LedBorg(RGBLED):
         each component of the LedBorg. If ``False``, construct regular
         :class:`LED` instances, which prevents smooth color graduations.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _PiBorg LedBorg: https://www.piborg.org/ledborg
     """
 
-    def __init__(self, initial_value=(0, 0, 0), pwm=True):
+    def __init__(self, initial_value=(0, 0, 0), pwm=True, pin_factory=None):
         super(LedBorg, self).__init__(red=17, green=27, blue=22,
-                                      pwm=pwm, initial_value=initial_value)
+                                      pwm=pwm, initial_value=initial_value,
+                                      pin_factory=pin_factory)
 
 
 class PiLiter(LEDBoard):
@@ -604,12 +658,17 @@ class PiLiter(LEDBoard):
         in when configured for output (warning: this can be on). If ``True``,
         the device will be switched on initially.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Ciseco Pi-LITEr: http://shop.ciseco.co.uk/pi-liter-8-led-strip-for-the-raspberry-pi/
     """
 
-    def __init__(self, pwm=False, initial_value=False):
+    def __init__(self, pwm=False, initial_value=False, pin_factory=None):
         super(PiLiter, self).__init__(4, 17, 27, 18, 22, 23, 24, 25,
-                                      pwm=pwm, initial_value=initial_value)
+                                      pwm=pwm, initial_value=initial_value,
+                                      pin_factory=pin_factory)
 
 
 class PiLiterBarGraph(LEDBarGraph):
@@ -634,13 +693,18 @@ class PiLiterBarGraph(LEDBarGraph):
         The initial :attr:`value` of the graph given as a float between -1 and
         +1. Defaults to ``0.0``.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Ciseco Pi-LITEr: http://shop.ciseco.co.uk/pi-liter-8-led-strip-for-the-raspberry-pi/
     """
 
-    def __init__(self, pwm=False, initial_value=0.0):
+    def __init__(self, pwm=False, initial_value=0.0, pin_factory=None):
         pins = (4, 17, 27, 18, 22, 23, 24, 25)
-        super(PiLiterBarGraph, self).__init__(*pins,
-                pwm=pwm, initial_value=initial_value)
+        super(PiLiterBarGraph, self).__init__(
+            *pins, pwm=pwm, initial_value=initial_value, pin_factory=pin_factory
+        )
 
 
 class TrafficLights(LEDBoard):
@@ -680,9 +744,14 @@ class TrafficLights(LEDBoard):
         The GPIO pin that the yellow LED is attached to. This is merely an
         alias for the ``amber`` parameter - you can't specify both ``amber``
         and ``yellow``.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
     """
     def __init__(self, red=None, amber=None, green=None,
-                 pwm=False, initial_value=False, yellow=None):
+                 pwm=False, initial_value=False, yellow=None,
+                 pin_factory=None):
         if amber is not None and yellow is not None:
             raise OutputDeviceBadValue(
                 'Only one of amber or yellow can be specified'
@@ -700,7 +769,7 @@ class TrafficLights(LEDBoard):
             )
         super(TrafficLights, self).__init__(
             pwm=pwm, initial_value=initial_value,
-            _order=devices.keys(),
+            _order=devices.keys(), pin_factory=pin_factory,
             **devices)
 
     def __getattr__(self, name):
@@ -710,17 +779,10 @@ class TrafficLights(LEDBoard):
             name = 'amber'
         return super(TrafficLights, self).__getattr__(name)
 
-    def __setattr__(self, name, value):
-        if name == 'amber' and self._display_yellow:
-            name = 'yellow'
-        elif name == 'yellow' and not self._display_yellow:
-            name = 'amber'
-        return super(TrafficLights, self).__setattr__(name, value)
-
 
 class PiTraffic(TrafficLights):
     """
-    Extends :class:`TrafficLights` for the `Low Voltage Labs PI-TRAFFIC`_:
+    Extends :class:`TrafficLights` for the `Low Voltage Labs PI-TRAFFIC`_
     vertical traffic lights board when attached to GPIO pins 9, 10, and 11.
 
     There's no need to specify the pins if the PI-TRAFFIC is connected to the
@@ -746,11 +808,188 @@ class PiTraffic(TrafficLights):
         in when configured for output (warning: this can be on). If ``True``,
         the device will be switched on initially.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Low Voltage Labs PI-TRAFFIC: http://lowvoltagelabs.com/products/pi-traffic/
     """
-    def __init__(self, pwm=False, initial_value=False):
+    def __init__(self, pwm=False, initial_value=False, pin_factory=None):
         super(PiTraffic, self).__init__(9, 10, 11,
-                                        pwm=pwm, initial_value=initial_value)
+                                        pwm=pwm, initial_value=initial_value,
+                                        pin_factory=pin_factory)
+
+
+class PiStop(TrafficLights):
+    """
+    Extends :class:`TrafficLights` for the `PiHardware Pi-Stop`_: a vertical
+    traffic lights board.
+
+    The following example turns on the amber LED on a Pi-Stop
+    connected to location ``A+``::
+
+        from gpiozero import PiStop
+
+        traffic = PiStop('A+')
+        traffic.amber.on()
+
+    :param str location:
+        The `location`_ on the GPIO header to which the Pi-Stop is connected.
+        Must be one of: ``A``, ``A+``, ``B``, ``B+``, ``C``, ``D``.
+
+    :param bool pwm:
+        If ``True``, construct :class:`PWMLED` instances to represent each
+        LED. If ``False`` (the default), construct regular :class:`LED`
+        instances.
+
+    :param bool initial_value:
+        If ``False`` (the default), all LEDs will be off initially. If
+        ``None``, each device will be left in whatever state the pin is found
+        in when configured for output (warning: this can be on). If ``True``,
+        the device will be switched on initially.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _PiHardware Pi-Stop: https://pihw.wordpress.com/meltwaters-pi-hardware-kits/pi-stop/
+    .. _location: https://github.com/PiHw/Pi-Stop/blob/master/markdown_source/markdown/Discover-PiStop.md
+    """
+    LOCATIONS = {
+        'A': (7, 8, 25),
+        'A+': (21, 20, 16),
+        'B': (10, 9, 11),
+        'B+': (13, 19, 26),
+        'C': (18, 15, 14),
+        'D': (2, 3, 4),
+    }
+
+    def __init__(
+            self, location=None, pwm=False, initial_value=False,
+            pin_factory=None):
+        gpios = self.LOCATIONS.get(location, None)
+        if gpios is None:
+            raise ValueError('location must be one of: %s' %
+                             ', '.join(sorted(self.LOCATIONS.keys())))
+        super(PiStop, self).__init__(
+            *gpios, pwm=pwm, initial_value=initial_value,
+            pin_factory=pin_factory
+        )
+
+
+class StatusZero(LEDBoard):
+    """
+    Extends :class:`LEDBoard` for The Pi Hut's `STATUS Zero`_: a Pi Zero sized
+    add-on board with three sets of red/green LEDs to provide a status
+    indicator.
+
+    The following example designates the first strip the label "wifi" and the
+    second "raining", and turns them green and red respectfully::
+
+        from gpiozero import StatusZero
+
+        status = StatusZero('wifi', 'raining')
+        status.wifi.green.on()
+        status.raining.red.on()
+
+    :param str \*labels:
+        Specify the names of the labels you wish to designate the strips to.
+        You can list up to three labels. If no labels are given, three strips
+        will be initialised with names 'one', 'two', and 'three'. If some, but
+        not all strips are given labels, any remaining strips will not be
+        initialised.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _STATUS Zero: https://thepihut.com/statuszero
+    """
+    default_labels = ('one', 'two', 'three')
+
+    def __init__(self, *labels, **kwargs):
+        pins = (
+            (17, 4),
+            (22, 27),
+            (9, 10),
+        )
+        pin_factory = kwargs.pop('pin_factory', None)
+        if len(labels) == 0:
+            labels = self.default_labels
+        elif len(labels) > len(pins):
+            raise ValueError("StatusZero doesn't support more than three labels")
+        dup, count = Counter(labels).most_common(1)[0]
+        if count > 1:
+            raise ValueError("Duplicate label %s" % dup)
+        super(StatusZero, self).__init__(
+            _order=labels, pin_factory=pin_factory, **{
+                label: LEDBoard(
+                    red=red, green=green, _order=('red', 'green'),
+                    pin_factory=pin_factory, **kwargs
+                )
+                for (green, red), label in zip(pins, labels)
+            }
+        )
+
+
+class StatusBoard(CompositeOutputDevice):
+    """
+    Extends :class:`CompositeOutputDevice` for The Pi Hut's `STATUS`_ board: a
+    HAT sized add-on board with five sets of red/green LEDs and buttons to
+    provide a status indicator with additional input.
+
+    The following example designates the first strip the label "wifi" and the
+    second "raining", turns the wifi green and then activates the button to
+    toggle its lights when pressed::
+
+        from gpiozero import StatusBoard
+
+        status = StatusBoard('wifi', 'raining')
+        status.wifi.lights.green.on()
+        status.wifi.button.when_pressed = status.wifi.lights.toggle
+
+    :param str \*labels:
+        Specify the names of the labels you wish to designate the strips to.
+        You can list up to five labels. If no labels are given, five strips
+        will be initialised with names 'one' to 'five'. If some, but not all
+        strips are given labels, any remaining strips will not be initialised.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _STATUS: https://thepihut.com/status
+    """
+    default_labels = ('one', 'two', 'three', 'four', 'five')
+
+    def __init__(self, *labels, **kwargs):
+        pins = (
+            (17, 4, 14),
+            (22, 27, 19),
+            (9, 10, 15),
+            (5, 11, 26),
+            (13, 6, 18),
+        )
+        pin_factory = kwargs.pop('pin_factory', None)
+        if len(labels) == 0:
+            labels = self.default_labels
+        elif len(labels) > len(pins):
+            raise ValueError("StatusBoard doesn't support more than five labels")
+        dup, count = Counter(labels).most_common(1)[0]
+        if count > 1:
+            raise ValueError("Duplicate label %s" % dup)
+        super(StatusBoard, self).__init__(
+            _order=labels, pin_factory=pin_factory, **{
+                label: CompositeOutputDevice(
+                    button=Button(button, pin_factory=pin_factory),
+                    lights=LEDBoard(
+                        red=red, green=green, _order=('red', 'green'),
+                        pin_factory=pin_factory, **kwargs
+                    ), _order=('button', 'lights'), pin_factory=pin_factory
+                )
+                for (green, red, button), label in zip(pins, labels)
+            }
+        )
 
 
 class SnowPi(LEDBoard):
@@ -779,30 +1018,39 @@ class SnowPi(LEDBoard):
         in when configured for output (warning: this can be on). If ``True``,
         the device will be switched on initially.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Ryanteck SnowPi: https://ryanteck.uk/raspberry-pi/114-snowpi-the-gpio-snowman-for-raspberry-pi-0635648608303.html
     """
-    def __init__(self, pwm=False, initial_value=False):
+    def __init__(self, pwm=False, initial_value=False, pin_factory=None):
         super(SnowPi, self).__init__(
             arms=LEDBoard(
                 left=LEDBoard(
                     top=17, middle=18, bottom=22,
                     pwm=pwm, initial_value=initial_value,
-                    _order=('top', 'middle', 'bottom')),
+                    _order=('top', 'middle', 'bottom'),
+                    pin_factory=pin_factory),
                 right=LEDBoard(
                     top=7, middle=8, bottom=9,
                     pwm=pwm, initial_value=initial_value,
-                    _order=('top', 'middle', 'bottom')),
-                _order=('left', 'right')
+                    _order=('top', 'middle', 'bottom'),
+                    pin_factory=pin_factory),
+                _order=('left', 'right'),
+                pin_factory=pin_factory
                 ),
             eyes=LEDBoard(
                 left=23, right=24,
                 pwm=pwm, initial_value=initial_value,
-                _order=('left', 'right')
+                _order=('left', 'right'),
+                pin_factory=pin_factory
                 ),
             nose=25,
             pwm=pwm, initial_value=initial_value,
-            _order=('eyes', 'nose', 'arms')
-            )
+            _order=('eyes', 'nose', 'arms'),
+            pin_factory=pin_factory
+        )
 
 
 class TrafficLightsBuzzer(CompositeOutputDevice):
@@ -819,12 +1067,18 @@ class TrafficLightsBuzzer(CompositeOutputDevice):
 
     :param Button button:
         An instance of :class:`Button` representing the button on the HAT.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
     """
 
-    def __init__(self, lights, buzzer, button):
+    def __init__(self, lights, buzzer, button, pin_factory=None):
         super(TrafficLightsBuzzer, self).__init__(
             lights=lights, buzzer=buzzer, button=button,
-            _order=('lights', 'buzzer', 'button'))
+            _order=('lights', 'buzzer', 'button'),
+            pin_factory=pin_factory
+        )
 
 
 class FishDish(TrafficLightsBuzzer):
@@ -847,14 +1101,19 @@ class FishDish(TrafficLightsBuzzer):
         LED. If ``False`` (the default), construct regular :class:`LED`
         instances.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Pi Supply FishDish: https://www.pi-supply.com/product/fish-dish-raspberry-pi-led-buzzer-board/
     """
 
-    def __init__(self, pwm=False):
+    def __init__(self, pwm=False, pin_factory=None):
         super(FishDish, self).__init__(
-            TrafficLights(9, 22, 4, pwm=pwm),
-            Buzzer(8),
-            Button(7, pull_up=False),
+            TrafficLights(9, 22, 4, pwm=pwm, pin_factory=pin_factory),
+            Buzzer(8, pin_factory=pin_factory),
+            Button(7, pull_up=False, pin_factory=pin_factory),
+            pin_factory=pin_factory
         )
 
 
@@ -878,14 +1137,19 @@ class TrafficHat(TrafficLightsBuzzer):
         LED. If ``False`` (the default), construct regular :class:`LED`
         instances.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Ryanteck Traffic HAT: https://ryanteck.uk/hats/1-traffichat-0635648607122.html
     """
 
-    def __init__(self, pwm=False):
+    def __init__(self, pwm=False, pin_factory=None):
         super(TrafficHat, self).__init__(
-            TrafficLights(24, 23, 22, pwm=pwm),
-            Buzzer(5),
-            Button(25),
+            TrafficLights(24, 23, 22, pwm=pwm, pin_factory=pin_factory),
+            Buzzer(5, pin_factory=pin_factory),
+            Button(25, pin_factory=pin_factory),
+            pin_factory=pin_factory
         )
 
 
@@ -911,13 +1175,19 @@ class Robot(SourceMixin, CompositeDevice):
     :param tuple right:
         A tuple of two GPIO pins representing the forward and backward inputs
         of the right motor's controller.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
     """
 
-    def __init__(self, left=None, right=None):
+    def __init__(self, left=None, right=None, pin_factory=None):
         super(Robot, self).__init__(
-                left_motor=Motor(*left),
-                right_motor=Motor(*right),
-                _order=('left_motor', 'right_motor'))
+            left_motor=Motor(*left, pin_factory=pin_factory),
+            right_motor=Motor(*right, pin_factory=pin_factory),
+            _order=('left_motor', 'right_motor'),
+            pin_factory=pin_factory
+        )
 
     @property
     def value(self):
@@ -933,6 +1203,216 @@ class Robot(SourceMixin, CompositeDevice):
     def value(self, value):
         self.left_motor.value, self.right_motor.value = value
 
+    def forward(self, speed=1, **kwargs):
+        """
+        Drive the robot forward by running both motors forward.
+
+        :param float speed:
+            Speed at which to drive the motors, as a value between 0 (stopped)
+            and 1 (full speed). The default is 1.
+
+        :param float curve_left:
+            The amount to curve left while moving forwards, by driving the
+            left motor at a slower speed. Maximum ``curve_left`` is 1, the
+            default is 0 (no curve). This parameter can only be specified as a
+            keyword parameter, and is mutually exclusive with ``curve_right``.
+
+        :param float curve_right:
+            The amount to curve right while moving forwards, by driving the
+            right motor at a slower speed. Maximum ``curve_right`` is 1, the
+            default is 0 (no curve). This parameter can only be specified as a
+            keyword parameter, and is mutually exclusive with ``curve_left``.
+        """
+        curve_left = kwargs.pop('curve_left', 0)
+        curve_right = kwargs.pop('curve_right', 0)
+        if kwargs:
+            raise TypeError('unexpected argument %s' % kwargs.popitem()[0])
+        if not 0 <= curve_left <= 1:
+            raise ValueError('curve_left must be between 0 and 1')
+        if not 0 <= curve_right <= 1:
+            raise ValueError('curve_right must be between 0 and 1')
+        if curve_left != 0 and curve_right != 0:
+            raise ValueError('curve_left and curve_right can\'t be used at the same time')
+        self.left_motor.forward(speed * (1 - curve_left))
+        self.right_motor.forward(speed * (1 - curve_right))
+
+    def backward(self, speed=1, **kwargs):
+        """
+        Drive the robot backward by running both motors backward.
+
+        :param float speed:
+            Speed at which to drive the motors, as a value between 0 (stopped)
+            and 1 (full speed). The default is 1.
+
+        :param float curve_left:
+            The amount to curve left while moving backwards, by driving the
+            left motor at a slower speed. Maximum ``curve_left`` is 1, the
+            default is 0 (no curve). This parameter can only be specified as a
+            keyword parameter, and is mutually exclusive with ``curve_right``.
+
+        :param float curve_right:
+            The amount to curve right while moving backwards, by driving the
+            right motor at a slower speed. Maximum ``curve_right`` is 1, the
+            default is 0 (no curve). This parameter can only be specified as a
+            keyword parameter, and is mutually exclusive with ``curve_left``.
+        """
+        curve_left = kwargs.pop('curve_left', 0)
+        curve_right = kwargs.pop('curve_right', 0)
+        if kwargs:
+            raise TypeError('unexpected argument %s' % kwargs.popitem()[0])
+        if not 0 <= curve_left <= 1:
+            raise ValueError('curve_left must be between 0 and 1')
+        if not 0 <= curve_right <= 1:
+            raise ValueError('curve_right must be between 0 and 1')
+        if curve_left != 0 and curve_right != 0:
+            raise ValueError('curve_left and curve_right can\'t be used at the same time')
+        self.left_motor.backward(speed * (1 - curve_left))
+        self.right_motor.backward(speed * (1 - curve_right))
+
+    def left(self, speed=1):
+        """
+        Make the robot turn left by running the right motor forward and left
+        motor backward.
+
+        :param float speed:
+            Speed at which to drive the motors, as a value between 0 (stopped)
+            and 1 (full speed). The default is 1.
+        """
+        self.right_motor.forward(speed)
+        self.left_motor.backward(speed)
+
+    def right(self, speed=1):
+        """
+        Make the robot turn right by running the left motor forward and right
+        motor backward.
+
+        :param float speed:
+            Speed at which to drive the motors, as a value between 0 (stopped)
+            and 1 (full speed). The default is 1.
+        """
+        self.left_motor.forward(speed)
+        self.right_motor.backward(speed)
+
+    def reverse(self):
+        """
+        Reverse the robot's current motor directions. If the robot is currently
+        running full speed forward, it will run full speed backward. If the
+        robot is turning left at half-speed, it will turn right at half-speed.
+        If the robot is currently stopped it will remain stopped.
+        """
+        self.left_motor.reverse()
+        self.right_motor.reverse()
+
+    def stop(self):
+        """
+        Stop the robot.
+        """
+        self.left_motor.stop()
+        self.right_motor.stop()
+
+
+class RyanteckRobot(Robot):
+    """
+    Extends :class:`Robot` for the `Ryanteck motor controller board`_.
+
+    The Ryanteck MCB pins are fixed and therefore there's no need to specify
+    them when constructing this class. The following example drives the robot
+    forward::
+
+        from gpiozero import RyanteckRobot
+
+        robot = RyanteckRobot()
+        robot.forward()
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _Ryanteck motor controller board: https://ryanteck.uk/add-ons/6-ryanteck-rpi-motor-controller-board-0635648607160.html
+    """
+
+    def __init__(self, pin_factory=None):
+        super(RyanteckRobot, self).__init__(
+            (17, 18), (22, 23), pin_factory=pin_factory
+        )
+
+
+class CamJamKitRobot(Robot):
+    """
+    Extends :class:`Robot` for the `CamJam #3 EduKit`_ motor controller board.
+
+    The CamJam robot controller pins are fixed and therefore there's no need
+    to specify them when constructing this class. The following example drives
+    the robot forward::
+
+        from gpiozero import CamJamKitRobot
+
+        robot = CamJamKitRobot()
+        robot.forward()
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _CamJam #3 EduKit: http://camjam.me/?page_id=1035
+    """
+
+    def __init__(self, pin_factory=None):
+        super(CamJamKitRobot, self).__init__(
+            (9, 10), (7, 8), pin_factory=pin_factory
+        )
+
+
+class PhaseEnableRobot(SourceMixin, CompositeDevice):
+    """
+    Extends :class:`CompositeDevice` to represent a dual-motor robot based
+    around a Phase/Enable motor board.
+
+    This class is constructed with two tuples representing the phase
+    (direction) and enable (speed) pins of the left and right controllers
+    respectively. For example, if the left motor's controller is connected to
+    GPIOs 12 and 5, while the right motor's controller is connected to GPIOs 13
+    and 6 so the following example will drive the robot forward::
+
+        from gpiozero import PhaseEnableRobot
+
+        robot = PhaseEnableRobot(left=(5, 12), right=(6, 13))
+        robot.forward()
+
+    :param tuple left:
+        A tuple of two GPIO pins representing the phase and enable inputs
+        of the left motor's controller.
+
+    :param tuple right:
+        A tuple of two GPIO pins representing the phase and enable inputs
+        of the right motor's controller.
+
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+    """
+
+    def __init__(self, left=None, right=None, pin_factory=None):
+        super(PhaseEnableRobot, self).__init__(
+            left_motor=PhaseEnableMotor(*left, pin_factory=pin_factory),
+            right_motor=PhaseEnableMotor(*right, pin_factory=pin_factory),
+            _order=('left_motor', 'right_motor'),
+            pin_factory=pin_factory
+        )
+
+    @property
+    def value(self):
+        """
+        Returns a tuple of two floating point values (-1 to 1) representing the
+        speeds of the robot's two motors (left and right). This property can
+        also be set to alter the speed of both motors.
+        """
+        return super(PhaseEnableRobot, self).value
+
+    @value.setter
+    def value(self, value):
+        self.left_motor.value, self.right_motor.value = value
+
     def forward(self, speed=1):
         """
         Drive the robot forward by running both motors forward.
@@ -986,8 +1466,8 @@ class Robot(SourceMixin, CompositeDevice):
         robot is turning left at half-speed, it will turn right at half-speed.
         If the robot is currently stopped it will remain stopped.
         """
-        self.left_motor.reverse()
-        self.right_motor.reverse()
+        self.left_motor.value = -self.left_motor.value
+        self.right_motor.value = -self.right_motor.value
 
     def stop(self):
         """
@@ -997,62 +1477,54 @@ class Robot(SourceMixin, CompositeDevice):
         self.right_motor.stop()
 
 
-class RyanteckRobot(Robot):
+class PololuDRV8835Robot(PhaseEnableRobot):
     """
-    Extends :class:`Robot` for the `Ryanteck MCB`_ robot.
+    Extends :class:`PhaseEnableRobot` for the `Pololu DRV8835 Dual Motor Driver
+    Kit`_.
 
-    The Ryanteck MCB pins are fixed and therefore there's no need to specify
+    The Pololu DRV8835 pins are fixed and therefore there's no need to specify
     them when constructing this class. The following example drives the robot
     forward::
 
-        from gpiozero import RyanteckRobot
+        from gpiozero import PololuDRV8835Robot
 
-        robot = RyanteckRobot()
+        robot = PololuDRV8835Robot()
         robot.forward()
 
-    .. _Ryanteck MCB: https://ryanteck.uk/add-ons/6-ryanteck-rpi-motor-controller-board-0635648607160.html
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
+    .. _Pololu DRV8835 Dual Motor Driver Kit: https://www.pololu.com/product/2753
     """
 
-    def __init__(self):
-        super(RyanteckRobot, self).__init__((17, 18), (22, 23))
-
-
-class CamJamKitRobot(Robot):
-    """
-    Extends :class:`Robot` for the `CamJam #3 EduKit`_ robot controller.
-
-    The CamJam robot controller pins are fixed and therefore there's no need
-    to specify them when constructing this class. The following example drives
-    the robot forward::
-
-        from gpiozero import CamJamKitRobot
-
-        robot = CamJamKitRobot()
-        robot.forward()
-
-    .. _CamJam #3 EduKit: http://camjam.me/?page_id=1035
-    """
-
-    def __init__(self):
-        super(CamJamKitRobot, self).__init__((9, 10), (7, 8))
+    def __init__(self, pin_factory=None):
+        super(PololuDRV8835Robot, self).__init__(
+            (5, 12), (6, 13), pin_factory=pin_factory
+        )
 
 
 class _EnergenieMaster(SharedMixin, CompositeOutputDevice):
-    def __init__(self):
+    def __init__(self, pin_factory=None):
         self._lock = Lock()
         super(_EnergenieMaster, self).__init__(
-                *(OutputDevice(pin) for pin in (17, 22, 23, 27)),
-                mode=OutputDevice(24), enable=OutputDevice(25),
-                _order=('mode', 'enable'))
+            *(
+                OutputDevice(pin, pin_factory=pin_factory)
+                for pin in (17, 22, 23, 27)
+            ),
+            mode=OutputDevice(24, pin_factory=pin_factory),
+            enable=OutputDevice(25, pin_factory=pin_factory),
+            _order=('mode', 'enable'), pin_factory=pin_factory
+        )
 
     def close(self):
-        if self._lock:
+        if getattr(self, '_lock', None):
             with self._lock:
                 super(_EnergenieMaster, self).close()
-            self._lock = None
+        self._lock = None
 
     @classmethod
-    def _shared_key(cls):
+    def _shared_key(cls, pin_factory):
         # There's only one Energenie master
         return None
 
@@ -1093,28 +1565,31 @@ class Energenie(SourceMixin, Device):
         the socket, which will be set upon construction. This defaults to
         ``False`` which will switch the socket off.
 
+    :param Factory pin_factory:
+        See :doc:`api_pins` for more information (this is an advanced feature
+        which most users can ignore).
+
     .. _Energenie socket: https://energenie4u.co.uk/index.php/catalogue/product/ENER002-2PI
     """
 
-    def __init__(self, socket=None, initial_value=False):
+    def __init__(self, socket=None, initial_value=False, pin_factory=None):
         if socket is None:
             raise EnergenieSocketMissing('socket number must be provided')
         if not (1 <= socket <= 4):
             raise EnergenieBadSocket('socket number must be between 1 and 4')
         self._value = None
-        super(Energenie, self).__init__()
+        super(Energenie, self).__init__(pin_factory=pin_factory)
         self._socket = socket
-        self._master = _EnergenieMaster()
+        self._master = _EnergenieMaster(pin_factory=pin_factory)
         if initial_value:
             self.on()
         else:
             self.off()
 
     def close(self):
-        if self._master:
-            m = self._master
-            self._master = None
-            m.close()
+        if getattr(self, '_master', None):
+            self._master.close()
+        self._master = None
 
     @property
     def closed(self):
@@ -1142,4 +1617,3 @@ class Energenie(SourceMixin, Device):
 
     def off(self):
         self.value = False
-
