@@ -9,7 +9,7 @@ str = type('')
 
 import inspect
 import weakref
-from functools import wraps
+from functools import wraps, partial
 from threading import Event
 from collections import deque
 from time import time
@@ -69,11 +69,14 @@ class SourceMixin(object):
         super(SourceMixin, self).__init__(*args, **kwargs)
 
     def close(self):
+        try:
+            self.source = None
+        except AttributeError:
+            pass
         try:
             super(SourceMixin, self).close()
         except AttributeError:
             pass
-        self.source = None
 
     def _copy_values(self, source):
         for v in source:
@@ -105,9 +108,9 @@ class SourceMixin(object):
 
     @source.setter
     def source(self, value):
-        if self._source_thread is not None:
+        if getattr(self, '_source_thread', None):
             self._source_thread.stop()
-            self._source_thread = None
+        self._source_thread = None
         self._source = value
         if value is not None:
             self._source_thread = GPIOThread(target=self._copy_values, args=(value,))
@@ -127,7 +130,7 @@ class SharedMixin(object):
     When :meth:`close` is called, an internal reference counter will be
     decremented and the instance will only close when it reaches zero.
     """
-    _INSTANCES = {}
+    _instances = {}
 
     def __del__(self):
         self._refs = 0
@@ -255,7 +258,16 @@ class EventsMixin(object):
             return None
         elif not callable(fn):
             raise BadEventHandler('value must be None or a callable')
-        elif inspect.isbuiltin(fn):
+        # If fn is wrapped with partial (i.e. partial, partialmethod, or wraps
+        # has been used to produce it) we need to dig out the "real" function
+        # that's been wrapped along with all the mandatory positional args
+        # used in the wrapper so we can test the binding
+        args = ()
+        wrapped_fn = fn
+        while isinstance(wrapped_fn, partial):
+            args = wrapped_fn.args + args
+            wrapped_fn = wrapped_fn.func
+        if inspect.isbuiltin(wrapped_fn):
             # We can't introspect the prototype of builtins. In this case we
             # assume that the builtin has no (mandatory) parameters; this is
             # the most reasonable assumption on the basis that pre-existing
@@ -267,13 +279,13 @@ class EventsMixin(object):
             # If this works, assume the function is capable of accepting no
             # parameters
             try:
-                inspect.getcallargs(fn)
+                inspect.getcallargs(wrapped_fn, *args)
                 return fn
             except TypeError:
                 try:
                     # If the above fails, try binding with a single parameter
                     # (ourselves). If this works, wrap the specified callback
-                    inspect.getcallargs(fn, self)
+                    inspect.getcallargs(wrapped_fn, *(args + (self,)))
                     @wraps(fn)
                     def wrapper():
                         return fn(self)
@@ -331,9 +343,9 @@ class HoldMixin(EventsMixin):
         self._hold_thread = HoldThread(self)
 
     def close(self):
-        if self._hold_thread:
+        if getattr(self, '_hold_thread', None):
             self._hold_thread.stop()
-            self._hold_thread = None
+        self._hold_thread = None
         try:
             super(HoldMixin, self).close()
         except AttributeError:
@@ -429,23 +441,28 @@ class HoldThread(GPIOThread):
     device is active.
     """
     def __init__(self, parent):
-        super(HoldThread, self).__init__(target=self.held, args=(parent,))
+        super(HoldThread, self).__init__(
+            target=self.held, args=(weakref.proxy(parent),))
         self.holding = Event()
         self.start()
 
     def held(self, parent):
-        while not self.stopping.is_set():
-            if self.holding.wait(0.1):
-                self.holding.clear()
-                while not (
-                        self.stopping.is_set() or
-                        parent._inactive_event.wait(parent.hold_time)
-                        ):
-                    if parent._held_from is None:
-                        parent._held_from = time()
-                    parent._fire_held()
-                    if not parent.hold_repeat:
-                        break
+        try:
+            while not self.stopping.is_set():
+                if self.holding.wait(0.1):
+                    self.holding.clear()
+                    while not (
+                            self.stopping.is_set() or
+                            parent._inactive_event.wait(parent.hold_time)
+                            ):
+                        if parent._held_from is None:
+                            parent._held_from = time()
+                        parent._fire_held()
+                        if not parent.hold_repeat:
+                            break
+        except ReferenceError:
+            # Parent is dead; time to die!
+            pass
 
 
 class GPIOQueue(GPIOThread):
@@ -484,15 +501,11 @@ class GPIOQueue(GPIOThread):
 
     def fill(self):
         try:
-            while (not self.stopping.wait(self.sample_wait) and
-                    len(self.queue) < self.queue.maxlen):
-                self.queue.append(self.parent._read())
-                if self.partial and isinstance(self.parent, EventsMixin):
-                    self.parent._fire_events()
-            self.full.set()
             while not self.stopping.wait(self.sample_wait):
                 self.queue.append(self.parent._read())
-                if isinstance(self.parent, EventsMixin):
+                if not self.full.is_set() and len(self.queue) >= self.queue.maxlen:
+                    self.full.set()
+                if (self.partial or self.full.is_set()) and isinstance(self.parent, EventsMixin):
                     self.parent._fire_events()
         except ReferenceError:
             # Parent is dead; time to die!
